import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;

import com.icl.saxon.*;
import com.icl.saxon.output.*;
import com.icl.saxon.expr.*;
import org.xml.sax.*;
 
public class GedServlet extends HttpServlet {

    /**
    * Respond to an HTTP request
    */
   
    public void service(HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
    {
        ServletOutputStream out = res.getOutputStream();

        try {
            String clear = req.getParameter("clear");
            if (clear!=null && clear.equals("yes")) {
                resetData();
            }

            String tree = req.getParameter("tree");
            String id = req.getParameter("id");

            PreparedStyleSheet style = getStyleSheet();
            DocumentInfo doc = getSourceDocument(tree);
        
            ParameterSet params = new ParameterSet();
            params.put("id", new StringValue(id));
            params.put("tree", new StringValue(tree));
        
            StyleSheetInstance transform = style.makeStyleSheetInstance();
        
            OutputDetails details = new OutputDetails();
            details.setOutputStream (out);
            transform.setOutputDetails(details);            
            transform.setParams(params);
            transform.renderDocument(doc, true);
           
        } catch (SAXException err) {
            out.println("Error applying stylesheet: " + err.getMessage());         
        } 
        res.setContentType("text/html");
    }

    /**
    * Get the prepared stylesheet from memory; prepare it if necessary
    */

    private synchronized PreparedStyleSheet getStyleSheet() throws SAXException {
        if (stylesheet == null) {
            stylesheet = new PreparedStyleSheet();
            File sheet = new File(getServletContext().getRealPath("/ged-servlet.xsl"));
            stylesheet.prepare(new ExtendedInputSource(sheet));
        }
        return stylesheet;
    }

    /**
    * Load the source document
    */

    private synchronized DocumentInfo getSourceDocument(String tree) throws SAXException {
        DocumentInfo doc = (DocumentInfo)trees.get(tree);
        if (doc==null) {
            File source = new File(getServletContext().getRealPath("/" + tree + ".xml"));
            doc = (new Builder()).build(new ExtendedInputSource(source));
        }
        return doc;
    }

    /**
    * Reset data held in memory
    */

    private synchronized void resetData() {
        trees = new Hashtable();
        stylesheet = null;
    }

    private Hashtable trees = new Hashtable();
    private PreparedStyleSheet stylesheet = null;
}
